﻿using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;

namespace UserService.DataModels
{
    public class OrganizationUnit : Node
    {
        public Member? Manager { get; set; }
    }

    public class SecurityGroup : Member
    {

    }

    public class User : Member
    {
        public string? FirstName { get; set; }
        public string? LastName { get; set; }
        public bool IsActive { get; set; }

        public string FullName => $"{FirstName} {LastName}";

        public IEnumerable<UserMember> MemberOf { get; set; } = new List<UserMember>();

        public User Clone()
        {
            return new User
            {
                Children = Children,
                CommonName = CommonName,
                Description = Description,
                EMail = EMail,
                FirstName = FirstName,
                Id = Id,
                IsActive = IsActive,
                LastName = LastName,
                MemberOf = MemberOf,
                Members = Members,
                Parent = Parent, 
                ParentId = ParentId
            };
        }
    }

    public class UserMember
    {
        public int MemberId { get; set; }
        public Member? Member { get; set; }

        public int UserId { get; set; }

        public User? User { get; set; }
    }

    public abstract class Member : Node
    {
        [EmailAddress]
        public string? EMail { get; set; }

        public ICollection<UserMember> Members { get; set; } = new List<UserMember>();
    }

    public abstract class Node
    {
        public int Id { get; set; }
        [Required] public string CommonName { get; set; } = "commonName";
        public string? Description { get; set; }
        public ICollection<Node> Children { get; set; } = new List<Node>();
        public Node? Parent { get; set; } //Parent
        public int? ParentId { get; set; }
    }
}