﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;
using UserService.DataModels;

namespace UserService.Repository
{
    public class OrganizationUnitRepository : IOrganizationUnitRepository
    {
        public async Task<IReadOnlyList<OrganizationUnit>> GetAllAsync(CancellationToken token = default)
        {
            await using var db = new UserServiceDbContext();
            return await db.OrganizationUnits.ToListAsync(token);
        }

        /// <inheritdoc />
        public async Task<OrganizationUnit?> GetAsync(Func<OrganizationUnit, bool> predicate,
            CancellationToken token = default)
        {
            await using var db = new UserServiceDbContext();
            return db.OrganizationUnits.Where(predicate).FirstOrDefault();
        }

        public async Task AddAsync(OrganizationUnit entity, CancellationToken token = default)
        {
            await using var db = new UserServiceDbContext();
            await db.OrganizationUnits.AddAsync(@entity, token);
            await db.SaveChangesAsync(token);
        }

        public async Task UpdateAsync(OrganizationUnit entity, CancellationToken token = default)
        {
            await using var db = new UserServiceDbContext();
            db.OrganizationUnits.Update(entity);
            await db.SaveChangesAsync(token);
        }

        public async Task DeleteAsync(OrganizationUnit entity, CancellationToken token = default)
        {
            await using var db = new UserServiceDbContext();
            db.OrganizationUnits.Remove(entity);
            await db.SaveChangesAsync(token);
        }
    }

    
    public class SecurityGroupsRepository : ISecurityGroupsRepository
    {
        public async Task<IReadOnlyList<SecurityGroup>> GetAllAsync(CancellationToken token = default)
        {
            await using var db = new UserServiceDbContext();
            return await db.SecurityGroups.ToListAsync(token);
        }

        /// <inheritdoc />
        public async Task<SecurityGroup?> GetAsync(Func<SecurityGroup, bool> predicate, CancellationToken token = default)
        {
            await using var db = new UserServiceDbContext();
            return db.SecurityGroups.Where(predicate).FirstOrDefault();
        }

        public async Task AddAsync(SecurityGroup entity, CancellationToken token = default)
        {
            await using var db = new UserServiceDbContext();
            await db.SecurityGroups.AddAsync(@entity, token);
            await db.SaveChangesAsync(token);
        }

        public async Task UpdateAsync(SecurityGroup entity, CancellationToken token = default)
        {
            await using var db = new UserServiceDbContext();
            db.SecurityGroups.Update(entity);
            await db.SaveChangesAsync(token);
        }

        public async Task DeleteAsync(SecurityGroup entity, CancellationToken token = default)
        {
            await using var db = new UserServiceDbContext();
            db.SecurityGroups.Remove(entity);
            await db.SaveChangesAsync(token);
        }
    }


    public class UsersRepository : IUsersRepository
    {
        public async Task<IReadOnlyList<User>> GetAllAsync(CancellationToken token = default)
        {
            await using var db = new UserServiceDbContext();
            return await db.Users.ToListAsync(token);
        }

        /// <inheritdoc />
        public async Task<User?> GetAsync(Func<User, bool> predicate, CancellationToken token = default)
        {
            await using var db = new UserServiceDbContext();
            return db.Users.Where(predicate).FirstOrDefault();
        }

        public async Task AddAsync(User entity, CancellationToken token = default)
        {
            await using var db = new UserServiceDbContext();
            await db.Users.AddAsync(@entity, token);
            await db.SaveChangesAsync(token);
        }

        public async Task UpdateAsync(User entity, CancellationToken token = default)
        {
            await using var db = new UserServiceDbContext();
            db.Users.Update(entity);
            await db.SaveChangesAsync(token);
        }

        public async Task DeleteAsync(User entity, CancellationToken token = default)
        {
            await using var db = new UserServiceDbContext();
            db.Users.Remove(entity);
            await db.SaveChangesAsync(token);
        }
    }
}