﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using UserService.DataModels;

namespace UserService.Repository
{
    public interface IRepository<T> where T : Node
    {
        Task<IReadOnlyList<T>> GetAllAsync(CancellationToken token = default);
        Task<T?> GetAsync(Func<T, bool> predicate, CancellationToken token = default);
        Task AddAsync(T entity, CancellationToken token = default);
        Task UpdateAsync(T entity, CancellationToken token = default);
        Task DeleteAsync(T entity, CancellationToken token = default);
    }

    public interface IOrganizationUnitRepository : IRepository<OrganizationUnit>
    {

    }

    public interface ISecurityGroupsRepository : IRepository<SecurityGroup>
    {

    }

    public interface IUsersRepository : IRepository<User>
    {

    }
}